@@ -8,13 +8,60 @@ from enum import Enum
 import pytest
 
 from voluptuous import (
-    ALLOW_EXTRA, PREVENT_EXTRA, All, AllInvalid, Any, Clamp, Coerce, Contains,
-    ContainsInvalid, Date, Datetime, Email, EmailInvalid, Equal, ExactSequence,
-    Exclusive, Extra, FqdnUrl, In, Inclusive, InInvalid, Invalid, IsDir, IsFile, Length,
-    Literal, LiteralInvalid, Marker, Match, MatchInvalid, Maybe, MultipleInvalid, NotIn,
-    NotInInvalid, Number, Object, Optional, PathExists, Range, Remove, Replace,
-    Required, Schema, Self, SomeOf, TooManyValid, TypeInvalid, Union, Unordered, Url,
-    UrlInvalid, raises, validate,
+    ALLOW_EXTRA,
+    PREVENT_EXTRA,
+    REMOVE_EXTRA,
+    All,
+    AllInvalid,
+    Any,
+    Clamp,
+    Coerce,
+    Contains,
+    ContainsInvalid,
+    Date,
+    Datetime,
+    Email,
+    EmailInvalid,
+    Equal,
+    ExactSequence,
+    Exclusive,
+    Extra,
+    FqdnUrl,
+    In,
+    Inclusive,
+    InInvalid,
+    Invalid,
+    IsDir,
+    IsFile,
+    Length,
+    Literal,
+    LiteralInvalid,
+    Marker,
+    Match,
+    MatchInvalid,
+    Maybe,
+    MultipleInvalid,
+    NotIn,
+    NotInInvalid,
+    Number,
+    Object,
+    Optional,
+    PathExists,
+    Range,
+    Remove,
+    Replace,
+    Required,
+    Schema,
+    Self,
+    SomeOf,
+    TooManyValid,
+    TypeInvalid,
+    Union,
+    Unordered,
+    Url,
+    UrlInvalid,
+    raises,
+    validate,
 )
 from voluptuous.humanize import humanize_error
 from voluptuous.util import Capitalize, Lower, Strip, Title, Upper
@@ -1704,7 +1751,7 @@ def test_key2():
     assert str(ctx.value.errors[1]) == "expecting a number @ data['four']"
 
 
-def test_key3():
+def test_any_with_extra_allow():
     schema = Schema(
         {
             Any("name", "area"): str,
@@ -1712,7 +1759,8 @@ def test_key3():
         },
         extra=ALLOW_EXTRA,
     )
-    schema(
+
+    result = schema(
         {
             "name": "one",
             "domain": "two",
@@ -1720,6 +1768,78 @@ def test_key3():
         }
     )
 
+    assert result == {
+        "name": "one",
+        "domain": "two",
+        "additional_key": "extra",
+    }
+
+
+def test_any_with_extra_remove():
+    schema = Schema(
+        {
+            Any("name", "area"): str,
+            "domain": str,
+        },
+        extra=REMOVE_EXTRA,
+    )
+
+    result = schema(
+        {
+            "name": "one",
+            "domain": "two",
+            "additional_key": "extra",
+        }
+    )
+
+    assert result == {
+        "name": "one",
+        "domain": "two",
+    }
+
+
+def test_any_with_extra_prevent():
+    schema = Schema(
+        {
+            Any("name", "area"): str,
+            "domain": str,
+        },
+        extra=PREVENT_EXTRA,
+    )
+
+    with pytest.raises(MultipleInvalid) as ctx:
+        schema(
+            {
+                "name": "one",
+                "domain": "two",
+                "additional_key": "extra",
+            }
+        )
+
+    assert len(ctx.value.errors) == 1
+    assert str(ctx.value.errors[0]) == "not a valid value @ data['additional_key']"
+
+
+def test_any_with_extra_none():
+    schema = Schema(
+        {
+            Any("name", "area"): str,
+            "domain": str,
+        },
+    )
+
+    with pytest.raises(MultipleInvalid) as ctx:
+        schema(
+            {
+                "name": "one",
+                "domain": "two",
+                "additional_key": "extra",
+            }
+        )
+
+    assert len(ctx.value.errors) == 1
+    assert str(ctx.value.errors[0]) == "not a valid value @ data['additional_key']"
+
 
 def test_coerce_enum():
     """Test Coerce Enum"""
@@ -1773,3 +1893,284 @@ def test_exception():
     assert str(ctx.value.errors) == f"[{invalid_scalar_excp_repr}]"
     ctx.value.add("Test Error")
     assert str(ctx.value.errors) == f"[{invalid_scalar_excp_repr}, 'Test Error']"
+
+
+# Additional tests for humanize.py module to improve coverage
+def test_humanize_error_with_nested_getitem_keyerror():
+    """Test _nested_getitem with KeyError (line 19-22)."""
+    from voluptuous.humanize import _nested_getitem
+
+    # Test KeyError handling
+    data = {'a': {'b': 1}}
+    path = ['a', 'c']  # 'c' doesn't exist in {'b': 1}
+    result = _nested_getitem(data, path)
+    assert result is None
+
+
+def test_humanize_error_with_nested_getitem_indexerror():
+    """Test _nested_getitem with IndexError (line 19-22)."""
+    from voluptuous.humanize import _nested_getitem
+
+    # Test IndexError handling
+    data = {'a': [1, 2, 3]}
+    path = ['a', 5]  # Index 5 doesn't exist in [1, 2, 3]
+    result = _nested_getitem(data, path)
+    assert result is None
+
+
+def test_humanize_error_with_nested_getitem_typeerror():
+    """Test _nested_getitem with TypeError (line 19-22)."""
+    from voluptuous.humanize import _nested_getitem
+
+    # Test TypeError handling - data is not subscriptable
+    data = 42  # int is not subscriptable
+    path = ['a']
+    result = _nested_getitem(data, path)
+    assert result is None
+
+
+def test_humanize_error_with_long_error_message():
+    """Test humanize_error with long error message that gets truncated (line 45)."""
+    from voluptuous.humanize import MAX_VALIDATION_ERROR_ITEM_LENGTH, humanize_error
+
+    # Create a very long string that will be truncated
+    long_string = "x" * (MAX_VALIDATION_ERROR_ITEM_LENGTH + 10)
+    data = {'a': long_string}
+    schema = Schema({'a': int})
+
+    with pytest.raises(MultipleInvalid) as ctx:
+        schema(data)
+
+    error_message = humanize_error(data, ctx.value, max_sub_error_length=50)
+    assert "..." in error_message
+    assert len(error_message.split("Got ")[1]) <= 53  # 50 + 3 for "..."
+
+
+def test_validate_with_humanized_errors_success():
+    """Test validate_with_humanized_errors with successful validation (line 54-57)."""
+    from voluptuous.humanize import validate_with_humanized_errors
+
+    schema = Schema({'a': int, 'b': str})
+    data = {'a': 42, 'b': 'hello'}
+
+    result = validate_with_humanized_errors(data, schema)
+    assert result == data
+
+
+def test_validate_with_humanized_errors_failure():
+    """Test validate_with_humanized_errors with validation failure (line 54-57)."""
+    from voluptuous.humanize import Error, validate_with_humanized_errors
+
+    schema = Schema({'a': int, 'b': str})
+    data = {'a': 'not an int', 'b': 123}
+
+    with pytest.raises(Error) as ctx:
+        validate_with_humanized_errors(data, schema)
+
+    error_message = str(ctx.value)
+    assert "expected int for dictionary value @ data['a']" in error_message
+    assert "expected str for dictionary value @ data['b']" in error_message
+    assert "Got 'not an int'" in error_message
+    assert "Got 123" in error_message
+
+
+def test_validate_with_humanized_errors_custom_max_length():
+    """Test validate_with_humanized_errors with custom max_sub_error_length."""
+    from voluptuous.humanize import Error, validate_with_humanized_errors
+
+    schema = Schema({'a': int})
+    data = {'a': 'not an int'}
+
+    with pytest.raises(Error) as ctx:
+        validate_with_humanized_errors(data, schema, max_sub_error_length=10)
+
+    error_message = str(ctx.value)
+    assert "..." in error_message  # Should be truncated
+
+
+def test_humanize_error_with_multiple_invalid():
+    """Test humanize_error with MultipleInvalid containing multiple errors."""
+    from voluptuous.humanize import humanize_error
+
+    schema = Schema({'a': int, 'b': str, 'c': [int]})
+    data = {'a': 'not an int', 'b': 123, 'c': ['not an int']}
+
+    with pytest.raises(MultipleInvalid) as ctx:
+        schema(data)
+
+    error_message = humanize_error(data, ctx.value)
+    # Should contain all three error messages
+    assert "expected int for dictionary value @ data['a']" in error_message
+    assert "expected str for dictionary value @ data['b']" in error_message
+    assert "expected int @ data['c'][0]" in error_message
+
+
+def test_humanize_error_with_single_invalid():
+    """Test humanize_error with single Invalid error."""
+    from voluptuous.humanize import humanize_error
+
+    schema = Schema({'a': int})
+    data = {'a': 'not an int'}
+
+    with pytest.raises(MultipleInvalid) as ctx:
+        schema(data)
+
+    error_message = humanize_error(data, ctx.value)
+    assert "expected int for dictionary value @ data['a']" in error_message
+    assert "Got 'not an int'" in error_message
+
+
+def test_humanize_error_with_none_data():
+    """Test humanize_error with None data."""
+    from voluptuous.humanize import _nested_getitem, humanize_error
+
+    # Test _nested_getitem with None data
+    result = _nested_getitem(None, ['a'])
+    assert result is None
+
+    # Test humanize_error with None data
+    schema = Schema({'a': int})
+    data = None
+
+    with pytest.raises(MultipleInvalid) as ctx:
+        schema(data)
+
+    error_message = humanize_error(data, ctx.value)
+    assert "expected a dictionary" in error_message
+
+
+def test_required_complex_key_any():
+    """Test Required with Any validator for multiple possible keys"""
+    schema = Schema(
+        {Required(Any("color", "temperature", "brightness")): str, "device_id": str}
+    )
+
+    # Should pass - defines one of the required keys
+    result = schema({"color": "red", "device_id": "light1"})
+    assert result == {"color": "red", "device_id": "light1"}
+
+    # Should pass - defines several of the required keys
+    result = schema({"color": "blue", "brightness": "50%", "device_id": "light1"})
+    assert result == {"color": "blue", "brightness": "50%", "device_id": "light1"}
+
+    # Should fail - has none of the required keys
+    with pytest.raises(MultipleInvalid) as ctx:
+        schema({"device_id": "light1"})
+
+    error_msg = str(ctx.value)
+    assert (
+        "at least one of ['color', 'temperature', 'brightness'] is required"
+        in error_msg
+    )
+
+
+def test_required_complex_key_custom_message():
+    """Test Required with Any validator and custom error message"""
+    schema = Schema(
+        {
+            Required(
+                Any("color", "temperature", "brightness"),
+                msg="Please specify a lighting attribute",
+            ): str,
+            "device_id": str,
+        }
+    )
+
+    # Should pass
+    schema({"color": "red", "device_id": "light1"})
+
+    # Should fail with custom message
+    with pytest.raises(MultipleInvalid) as ctx:
+        schema({"device_id": "light1"})
+
+    error_msg = str(ctx.value)
+    assert "Please specify a lighting attribute" in error_msg
+
+
+def test_required_complex_key_mixed_types():
+    """Test Required with Any validator containing mixed key types"""
+    schema = Schema({Required(Any("string_key", 123, 45.6)): str, "other": int})
+
+    # Should work with string key
+    result = schema({"string_key": "value", "other": 1})
+    assert result == {"string_key": "value", "other": 1}
+
+    # Should work with int key
+    result = schema({123: "value", "other": 1})
+    assert result == {123: "value", "other": 1}
+
+    # Should work with float key
+    result = schema({45.6: "value", "other": 1})
+    assert result == {45.6: "value", "other": 1}
+
+    # Should fail with none present
+    with pytest.raises(MultipleInvalid) as ctx:
+        schema({"other": 1})
+
+    error_msg = str(ctx.value)
+    assert "at least one of ['string_key', 123, 45.6] is required" in error_msg
+
+
+def test_required_complex_key_multiple_complex_requirements():
+    """Test multiple Required complex keys in same schema"""
+    schema = Schema(
+        {
+            Required(Any("color", "hue")): str,
+            Required(Any("brightness", "intensity")): str,
+            "device": str,
+        }
+    )
+
+    # Should pass with one from each group
+    result = schema({"color": "red", "brightness": "high", "device": "light"})
+    assert result == {"color": "red", "brightness": "high", "device": "light"}
+
+    # Should fail if missing on any group
+    with pytest.raises(MultipleInvalid) as ctx:
+        schema({"brightness": "high", "device": "light"})
+
+    error_msg = str(ctx.value)
+    assert "at least one of ['color', 'hue'] is required" in error_msg
+
+
+def test_required_complex_key_value_validation():
+    """Test that value validation still works with complex required keys"""
+    schema = Schema({Required(Any("color", "temperature")): str, "device": str})
+
+    # Should pass with valid string value
+    result = schema({"color": "red", "device": "light"})
+    assert result == {"color": "red", "device": "light"}
+
+    # Should fail with invalid value type
+    with pytest.raises(MultipleInvalid) as ctx:
+        schema({"color": 123, "device": "light"})  # color should be str, not int
+
+    error_msg = str(ctx.value)
+    assert "expected str" in error_msg
+
+
+def test_complex_required_keys_with_specific_value_validation():
+    """Test complex required keys combined with specific value validation for brightness range."""
+    schema = Schema(
+        {
+            Required(Any('color', 'temperature', 'brightness')): object,
+            'brightness': All(
+                Coerce(int), Range(min=0, max=100)
+            ),  # Additional validation for brightness specifically
+            'device_id': str,
+        }
+    )
+
+    # Valid - color provided, no brightness validation needed
+    result = schema({'color': 'red', 'device_id': 'light1'})
+    assert result == {'color': 'red', 'device_id': 'light1'}
+
+    # Invalid - brightness provided but out of range (255 > 100)
+    # Should NOT get "required field missing" error, but should get range error
+    with pytest.raises(MultipleInvalid) as exc_info:
+        schema({'brightness': '255', 'device_id': 'light1'})
+
+    # Verify it's a range error, not a missing required field error
+    error_msg = str(exc_info.value)
+    assert "required" not in error_msg.lower()  # No "required field missing" error
+    assert "value must be at most 100" in error_msg  # Range validation error
